@@ -233,7 +233,7 @@ update_chassis_transport_zones(const struct sset *transport_zones,
  * Parse an ovs 'encap_type' string and stores the resulting types in the
  * 'encap_type_set' string set.
  */
-static bool
+static void
 chassis_parse_ovs_encap_type(const char *encap_type,
                              struct sset *encap_type_set)
 {
@@ -247,26 +247,23 @@ chassis_parse_ovs_encap_type(const char *encap_type,
             VLOG_INFO_RL(&rl, "Unknown tunnel type: %s", type);
         }
     }
-
-    return true;
 }
 
 /*
  * Parse an ovs 'encap_ip' string and stores the resulting IP representations
  * in the 'encap_ip_set' string set.
  */
-static bool
+static void
 chassis_parse_ovs_encap_ip(const char *encap_ip, struct sset *encap_ip_set)
 {
     sset_from_delimited_string(encap_ip_set, encap_ip, ",");
-    return true;
 }
 
 /*
  * Parse the ovs 'iface_types' and store them in the format required by the
  * Chassis record.
  */
-static bool
+static void
 chassis_parse_ovs_iface_types(char **iface_types, size_t n_iface_types,
                               struct ds *iface_types_str)
 {
@@ -274,7 +271,6 @@ chassis_parse_ovs_iface_types(char **iface_types, size_t n_iface_types,
         ds_put_format(iface_types_str, "%s,", iface_types[i]);
     }
     ds_chomp(iface_types_str, ',');
-    return true;
 }
 
 /*
@@ -329,26 +325,17 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
     ovs_cfg->trim_timeout_ms =
         get_trim_timeout(&cfg->external_ids, chassis_id);
 
-    if (!chassis_parse_ovs_encap_type(encap_type, &ovs_cfg->encap_type_set)) {
-        return false;
-    }
+    chassis_parse_ovs_encap_type(encap_type, &ovs_cfg->encap_type_set);
 
     /* 'ovn-encap-ip' can accept a comma-delimited list of IP addresses instead
      * of a single IP address. Although this is undocumented, it can be used
      * to enable certain hardware-offloaded use cases in which a host has
      * multiple NICs and is assigning SR-IOV VFs to a guest (as logical ports).
      */
-    if (!chassis_parse_ovs_encap_ip(encap_ips, &ovs_cfg->encap_ip_set)) {
-        sset_destroy(&ovs_cfg->encap_type_set);
-        return false;
-    }
+    chassis_parse_ovs_encap_ip(encap_ips, &ovs_cfg->encap_ip_set);
 
-    if (!chassis_parse_ovs_iface_types(cfg->iface_types,
-                                       cfg->n_iface_types,
-                                       &ovs_cfg->iface_types)) {
-        sset_destroy(&ovs_cfg->encap_type_set);
-        sset_destroy(&ovs_cfg->encap_ip_set);
-    }
+    chassis_parse_ovs_iface_types(cfg->iface_types, cfg->n_iface_types,
+                                  &ovs_cfg->iface_types);
 
     ovs_cfg->is_interconn = get_is_interconn(&cfg->external_ids, chassis_id);
 
@@ -381,6 +368,9 @@ chassis_build_other_config(const struct ovs_chassis_cfg *ovs_cfg,
     smap_replace(config, OVN_FEATURE_CT_NO_MASKED_LABEL, "true");
     smap_replace(config, OVN_FEATURE_MAC_BINDING_TIMESTAMP, "true");
     smap_replace(config, OVN_FEATURE_CT_LB_RELATED, "true");
+    smap_replace(config, OVN_FEATURE_FDB_TIMESTAMP, "true");
+    smap_replace(config, OVN_FEATURE_LS_DPG_COLUMN, "true");
+    smap_replace(config, OVN_FEATURE_CT_COMMIT_NAT_V2, "true");
 }
 
 /*
@@ -508,6 +498,24 @@ chassis_other_config_changed(const struct ovs_chassis_cfg *ovs_cfg,
         return true;
     }
 
+    if (!smap_get_bool(&chassis_rec->other_config,
+                       OVN_FEATURE_FDB_TIMESTAMP,
+                       false)) {
+        return true;
+    }
+
+    if (!smap_get_bool(&chassis_rec->other_config,
+                       OVN_FEATURE_LS_DPG_COLUMN,
+                       false)) {
+        return true;
+    }
+
+    if (!smap_get_bool(&chassis_rec->other_config,
+                       OVN_FEATURE_CT_COMMIT_NAT_V2,
+                       false)) {
+        return true;
+    }
+
     return false;
 }
 
@@ -610,6 +618,58 @@ chassis_build_encaps(struct ovsdb_idl_txn *ovnsb_idl_txn,
     return encaps;
 }
 
+/* 'struct sset' of all supported options in other_confing. Anything missing
+ * in this set will be removed from the chassis configuration. */
+static void
+update_supported_sset(struct sset *supported)
+{
+    /* OvS external_ids. */
+    sset_add(supported, "ovn-bridge-mappings");
+    sset_add(supported, "datapath-type");
+    sset_add(supported, "ovn-cms-options");
+    sset_add(supported, "ovn-monitor-all");
+    sset_add(supported, "ovn-enable-lflow-cache");
+    sset_add(supported, "ovn-limit-lflow-cache");
+    sset_add(supported, "ovn-memlimit-lflow-cache-kb");
+    sset_add(supported, "ovn-trim-limit-lflow-cache");
+    sset_add(supported, "ovn-trim-wmark-perc-lflow-cache");
+    sset_add(supported, "ovn-trim-timeout-ms");
+    sset_add(supported, "iface-types");
+    sset_add(supported, "ovn-chassis-mac-mappings");
+    sset_add(supported, "is-interconn");
+
+    /* Internal options. */
+    sset_add(supported, "is-vtep");
+    sset_add(supported, "is-remote");
+    sset_add(supported, OVN_FEATURE_PORT_UP_NOTIF);
+    sset_add(supported, OVN_FEATURE_CT_NO_MASKED_LABEL);
+    sset_add(supported, OVN_FEATURE_MAC_BINDING_TIMESTAMP);
+    sset_add(supported, OVN_FEATURE_CT_LB_RELATED);
+    sset_add(supported, OVN_FEATURE_FDB_TIMESTAMP);
+    sset_add(supported, OVN_FEATURE_LS_DPG_COLUMN);
+    sset_add(supported, OVN_FEATURE_CT_COMMIT_NAT_V2);
+}
+
+static void
+remove_unsupported_options(const struct sbrec_chassis *chassis_rec,
+                           bool *updated)
+{
+    struct sset supported_options = SSET_INITIALIZER(&supported_options);
+    update_supported_sset(&supported_options);
+
+    const struct smap_node *node;
+    SMAP_FOR_EACH (node, &chassis_rec->other_config) {
+        if (!sset_contains(&supported_options, node->key)) {
+            VLOG_WARN("Removing unsupported key \"%s\" from chassis record.",
+                      node->key);
+            sbrec_chassis_update_other_config_delkey(chassis_rec, node->key);
+            *updated = true;
+        }
+    }
+
+    sset_destroy(&supported_options);
+}
+
 /* If this is a chassis config update after we initialized the record once
  * then we should always be able to find it with the ID we saved in
  * chassis_state.
@@ -675,6 +735,8 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
 
     update_chassis_transport_zones(transport_zones, chassis_rec);
 
+    remove_unsupported_options(chassis_rec, &updated);
+
     /* If any of the encaps should change, update them. */
     bool tunnels_changed =
         chassis_tunnels_changed(&ovs_cfg->encap_type_set,
@@ -714,7 +776,7 @@ chassis_private_get_record(
             chassis_private_lookup_by_name(sbrec_chassis_pvt_by_name,
                                            chassis_id);
 
-    if (!chassis_p && ovnsb_idl_txn) {
+    if (!chassis_p) {
         return sbrec_chassis_private_insert(ovnsb_idl_txn);
     }
 
