@@ -33,34 +33,21 @@
 
 VLOG_DEFINE_THIS_MODULE(en_sync_to_sb);
 
-/* This is just a type wrapper to enforce that it has to be sorted. */
-struct sorted_addresses {
-    const char **arr;
-    size_t n;
-};
-
-
 static void sync_addr_set(struct ovsdb_idl_txn *ovnsb_txn, const char *name,
-                          struct sorted_addresses *addresses,
+                          struct sorted_array *addresses,
                           struct shash *sb_address_sets);
-static void sync_addr_sets(const struct nbrec_address_set_table *,
+static void sync_addr_sets(struct ovsdb_idl_txn *ovnsb_txn,
+                           const struct nbrec_address_set_table *,
                            const struct nbrec_port_group_table *,
                            const struct sbrec_address_set_table *,
-                           struct ovsdb_idl_txn *ovnsb_txn,
-                           struct hmap *datapaths);
+                           const struct ovn_datapaths *lr_datapaths);
 static const struct sbrec_address_set *sb_address_set_lookup_by_name(
     struct ovsdb_idl_index *, const char *name);
-static void update_sb_addr_set(struct sorted_addresses *,
+static void update_sb_addr_set(struct sorted_array *,
                                const struct sbrec_address_set *);
 static void build_port_group_address_set(const struct nbrec_port_group *,
                                          struct svec *ipv4_addrs,
                                          struct svec *ipv6_addrs);
-static struct sorted_addresses
-sorted_addresses_from_nbrec(const struct nbrec_address_set *nb_as);
-static struct sorted_addresses
-sorted_addresses_from_svec(struct svec *addresses);
-static struct sorted_addresses
-sorted_addresses_from_sset(struct sset *addresses);
 
 void *
 en_sync_to_sb_init(struct engine_node *node OVS_UNUSED,
@@ -101,9 +88,9 @@ en_sync_to_sb_addr_set_run(struct engine_node *node, void *data OVS_UNUSED)
     const struct engine_context *eng_ctx = engine_get_context();
     struct northd_data *northd_data = engine_get_input_data("northd", node);
 
-    sync_addr_sets(nb_address_set_table, nb_port_group_table,
-                   sb_address_set_table, eng_ctx->ovnsb_idl_txn,
-                   &northd_data->datapaths);
+    sync_addr_sets(eng_ctx->ovnsb_idl_txn, nb_address_set_table,
+                   nb_port_group_table, sb_address_set_table,
+                   &northd_data->lr_datapaths);
 
     engine_set_node_state(node, EN_UPDATED);
 }
@@ -145,9 +132,10 @@ sync_to_sb_addr_set_nb_address_set_handler(struct engine_node *node,
         if (!sb_addr_set) {
             return false;
         }
-        struct sorted_addresses addrs =
-                sorted_addresses_from_nbrec(nb_addr_set);
+        struct sorted_array addrs =
+            sorted_array_from_dbrec(nb_addr_set, addresses);
         update_sb_addr_set(&addrs, sb_addr_set);
+        sorted_array_destroy(&addrs);
     }
 
     return true;
@@ -194,27 +182,146 @@ sync_to_sb_addr_set_nb_port_group_handler(struct engine_node *node,
         struct svec ipv6_addrs = SVEC_EMPTY_INITIALIZER;
         build_port_group_address_set(nb_pg, &ipv4_addrs, &ipv6_addrs);
 
-        struct sorted_addresses ipv4_addrs_sorted =
-                sorted_addresses_from_svec(&ipv4_addrs);
-        struct sorted_addresses ipv6_addrs_sorted =
-                sorted_addresses_from_svec(&ipv6_addrs);
+        struct sorted_array ipv4_addrs_sorted =
+                sorted_array_from_svec(&ipv4_addrs);
+        struct sorted_array ipv6_addrs_sorted =
+                sorted_array_from_svec(&ipv6_addrs);
 
         update_sb_addr_set(&ipv4_addrs_sorted, sb_addr_set_v4);
         update_sb_addr_set(&ipv6_addrs_sorted, sb_addr_set_v6);
 
-        free(ipv4_addrs_name);
-        free(ipv6_addrs_name);
+        sorted_array_destroy(&ipv4_addrs_sorted);
+        sorted_array_destroy(&ipv6_addrs_sorted);
         svec_destroy(&ipv4_addrs);
         svec_destroy(&ipv6_addrs);
+        free(ipv4_addrs_name);
+        free(ipv6_addrs_name);
     }
 
     return true;
 }
 
+/* sync_to_sb_lb engine node functions.
+ * This engine node syncs the SB load balancers.
+ */
+void *
+en_sync_to_sb_lb_init(struct engine_node *node OVS_UNUSED,
+                      struct engine_arg *arg OVS_UNUSED)
+{
+    return NULL;
+}
+
+void
+en_sync_to_sb_lb_run(struct engine_node *node, void *data OVS_UNUSED)
+{
+    const struct sbrec_load_balancer_table *sb_load_balancer_table =
+        EN_OVSDB_GET(engine_get_input("SB_load_balancer", node));
+    const struct engine_context *eng_ctx = engine_get_context();
+    struct northd_data *northd_data = engine_get_input_data("northd", node);
+
+    sync_lbs(eng_ctx->ovnsb_idl_txn, sb_load_balancer_table,
+             &northd_data->ls_datapaths, &northd_data->lr_datapaths,
+             &northd_data->lb_datapaths_map, &northd_data->features);
+    engine_set_node_state(node, EN_UPDATED);
+}
+
+void
+en_sync_to_sb_lb_cleanup(void *data OVS_UNUSED)
+{
+
+}
+
+bool
+sync_to_sb_lb_northd_handler(struct engine_node *node, void *data OVS_UNUSED)
+{
+    struct northd_data *nd = engine_get_input_data("northd", node);
+
+    if (!nd->change_tracked || nd->lb_changed) {
+        /* Return false if no tracking data or if lbs changed. */
+        return false;
+    }
+
+
+    /* There are only NB LSP related changes and these can be safely
+     * ignore and returned true.  However in case the northd engine
+     * tracking data includes other changes, we need to do additional
+     * checks before safely ignoring. */
+    return true;
+}
+
+bool
+sync_to_sb_lb_sb_load_balancer(struct engine_node *node, void *data OVS_UNUSED)
+{
+    const struct sbrec_load_balancer_table *sb_load_balancer_table =
+        EN_OVSDB_GET(engine_get_input("SB_load_balancer", node));
+
+    /* The only reason to handle SB.Load_Balancer updates is to detect
+     * spurious records being created in clustered databases due to
+     * lack of indexing on the SB.Load_Balancer table.  All other changes
+     * are valid and performed by northd, the only write-client for
+     * this table. */
+    if (check_sb_lb_duplicates(sb_load_balancer_table)) {
+        return false;
+    }
+    return true;
+}
+
+/* sync_to_sb_pb engine node functions.
+ * This engine node syncs the SB Port Bindings (partly).
+ * en_northd engine create the SB Port binding rows and
+ * updates most of the columns.
+ * This engine node updates the port binding columns which
+ * needs to be updated after northd engine is run.
+ */
+
+void *
+en_sync_to_sb_pb_init(struct engine_node *node OVS_UNUSED,
+                      struct engine_arg *arg OVS_UNUSED)
+{
+    return NULL;
+}
+
+void
+en_sync_to_sb_pb_run(struct engine_node *node, void *data OVS_UNUSED)
+{
+    const struct engine_context *eng_ctx = engine_get_context();
+    struct northd_data *northd_data = engine_get_input_data("northd", node);
+
+    sync_pbs(eng_ctx->ovnsb_idl_txn, &northd_data->ls_ports);
+    engine_set_node_state(node, EN_UPDATED);
+}
+
+void
+en_sync_to_sb_pb_cleanup(void *data OVS_UNUSED)
+{
+
+}
+
+bool
+sync_to_sb_pb_northd_handler(struct engine_node *node, void *data OVS_UNUSED)
+{
+    const struct engine_context *eng_ctx = engine_get_context();
+    if (!eng_ctx->ovnsb_idl_txn) {
+        return false;
+    }
+
+    struct northd_data *nd = engine_get_input_data("northd", node);
+    if (!nd->change_tracked) {
+        return false;
+    }
+
+    if (!sync_pbs_for_northd_ls_changes(&nd->tracked_ls_changes)) {
+        return false;
+    }
+
+    engine_set_node_state(node, EN_UPDATED);
+    return true;
+}
+
 /* static functions. */
 static void
 sync_addr_set(struct ovsdb_idl_txn *ovnsb_txn, const char *name,
-              struct sorted_addresses *addresses,
+              struct sorted_array *addresses,
               struct shash *sb_address_sets)
 {
     const struct sbrec_address_set *sb_address_set;
@@ -245,10 +352,11 @@ sync_addr_set(struct ovsdb_idl_txn *ovnsb_txn, const char *name,
  * in OVN_Northbound, so that the address sets used in Logical_Flows in
  * OVN_Southbound is checked against the proper set.*/
 static void
-sync_addr_sets(const struct nbrec_address_set_table *nb_address_set_table,
+sync_addr_sets(struct ovsdb_idl_txn *ovnsb_txn,
+               const struct nbrec_address_set_table *nb_address_set_table,
                const struct nbrec_port_group_table *nb_port_group_table,
                const struct sbrec_address_set_table *sb_address_set_table,
-               struct ovsdb_idl_txn *ovnsb_txn, struct hmap *datapaths)
+               const struct ovn_datapaths *lr_datapaths)
 {
     struct shash sb_address_sets = SHASH_INITIALIZER(&sb_address_sets);
 
@@ -260,11 +368,9 @@ sync_addr_sets(const struct nbrec_address_set_table *nb_address_set_table,
 
     /* Service monitor MAC. */
     const char *svc_monitor_macp = northd_get_svc_monitor_mac();
-    struct sorted_addresses svc = {
-            .arr = &svc_monitor_macp,
-            .n = 1,
-    };
+    struct sorted_array svc = sorted_array_create(&svc_monitor_macp, 1, false);
     sync_addr_set(ovnsb_txn, "svc_monitor_mac", &svc, &sb_address_sets);
+    sorted_array_destroy(&svc);
 
     /* sync port group generated address sets first */
     const struct nbrec_port_group *nb_port_group;
@@ -276,50 +382,50 @@ sync_addr_sets(const struct nbrec_address_set_table *nb_address_set_table,
         char *ipv4_addrs_name = xasprintf("%s_ip4", nb_port_group->name);
         char *ipv6_addrs_name = xasprintf("%s_ip6", nb_port_group->name);
 
-        struct sorted_addresses ipv4_addrs_sorted =
-                sorted_addresses_from_svec(&ipv4_addrs);
-        struct sorted_addresses ipv6_addrs_sorted =
-                sorted_addresses_from_svec(&ipv6_addrs);
+        struct sorted_array ipv4_addrs_sorted =
+                sorted_array_from_svec(&ipv4_addrs);
+        struct sorted_array ipv6_addrs_sorted =
+                sorted_array_from_svec(&ipv6_addrs);
 
         sync_addr_set(ovnsb_txn, ipv4_addrs_name,
                       &ipv4_addrs_sorted, &sb_address_sets);
         sync_addr_set(ovnsb_txn, ipv6_addrs_name,
                       &ipv6_addrs_sorted, &sb_address_sets);
-        free(ipv4_addrs_name);
-        free(ipv6_addrs_name);
+        sorted_array_destroy(&ipv4_addrs_sorted);
+        sorted_array_destroy(&ipv6_addrs_sorted);
         svec_destroy(&ipv4_addrs);
         svec_destroy(&ipv6_addrs);
+        free(ipv4_addrs_name);
+        free(ipv6_addrs_name);
     }
 
     /* Sync router load balancer VIP generated address sets. */
     struct ovn_datapath *od;
-    HMAP_FOR_EACH (od, key_node, datapaths) {
-        if (!od->nbr) {
-            continue;
-        }
+    HMAP_FOR_EACH (od, key_node, &lr_datapaths->datapaths) {
+        ovs_assert(od->nbr);
 
         if (sset_count(&od->lb_ips->ips_v4_reachable)) {
             char *ipv4_addrs_name = lr_lb_address_set_name(od->tunnel_key,
                                                            AF_INET);
 
-            struct sorted_addresses ipv4_addrs_sorted =
-                    sorted_addresses_from_sset(&od->lb_ips->ips_v4_reachable);
+            struct sorted_array ipv4_addrs_sorted =
+                    sorted_array_from_sset(&od->lb_ips->ips_v4_reachable);
 
             sync_addr_set(ovnsb_txn, ipv4_addrs_name,
                           &ipv4_addrs_sorted, &sb_address_sets);
-            free(ipv4_addrs_sorted.arr);
+            sorted_array_destroy(&ipv4_addrs_sorted);
             free(ipv4_addrs_name);
         }
 
         if (sset_count(&od->lb_ips->ips_v6_reachable)) {
             char *ipv6_addrs_name = lr_lb_address_set_name(od->tunnel_key,
                                                            AF_INET6);
-            struct sorted_addresses ipv6_addrs_sorted =
-                    sorted_addresses_from_sset(&od->lb_ips->ips_v6_reachable);
+            struct sorted_array ipv6_addrs_sorted =
+                    sorted_array_from_sset(&od->lb_ips->ips_v6_reachable);
 
             sync_addr_set(ovnsb_txn, ipv6_addrs_name,
                           &ipv6_addrs_sorted, &sb_address_sets);
-            free(ipv6_addrs_sorted.arr);
+            sorted_array_destroy(&ipv6_addrs_sorted);
             free(ipv6_addrs_name);
         }
     }
@@ -329,10 +435,11 @@ sync_addr_sets(const struct nbrec_address_set_table *nb_address_set_table,
     const struct nbrec_address_set *nb_address_set;
     NBREC_ADDRESS_SET_TABLE_FOR_EACH (nb_address_set,
                                       nb_address_set_table) {
-        struct sorted_addresses addrs =
-                sorted_addresses_from_nbrec(nb_address_set);
+        struct sorted_array addrs =
+                sorted_array_from_dbrec(nb_address_set, addresses);
         sync_addr_set(ovnsb_txn, nb_address_set->name,
                       &addrs, &sb_address_sets);
+        sorted_array_destroy(&addrs);
     }
 
     struct shash_node *node;
@@ -344,39 +451,25 @@ sync_addr_sets(const struct nbrec_address_set_table *nb_address_set_table,
 }
 
 static void
-update_sb_addr_set(struct sorted_addresses *nb_addresses,
+sb_addr_set_apply_diff(const void *arg, const char *item, bool add)
+{
+    const struct sbrec_address_set *as = arg;
+    if (add) {
+        sbrec_address_set_update_addresses_addvalue(as, item);
+    } else {
+        sbrec_address_set_update_addresses_delvalue(as, item);
+    }
+}
+
+static void
+update_sb_addr_set(struct sorted_array *nb_addresses,
                    const struct sbrec_address_set *sb_as)
 {
-    size_t nb_index, sb_index;
-
-    const char **nb_arr = nb_addresses->arr;
-    char **sb_arr = sb_as->addresses;
-    size_t nb_n = nb_addresses->n;
-    size_t sb_n = sb_as->n_addresses;
-
-    for (nb_index = sb_index = 0; nb_index < nb_n && sb_index < sb_n;) {
-        int cmp = strcmp(nb_arr[nb_index], sb_arr[sb_index]);
-        if (cmp < 0) {
-            sbrec_address_set_update_addresses_addvalue(sb_as,
-                                                        nb_arr[nb_index]);
-            nb_index++;
-        } else if (cmp > 0) {
-            sbrec_address_set_update_addresses_delvalue(sb_as,
-                                                        sb_arr[sb_index]);
-            sb_index++;
-        } else {
-            nb_index++;
-            sb_index++;
-        }
-    }
-
-    for (; nb_index < nb_n; nb_index++) {
-        sbrec_address_set_update_addresses_addvalue(sb_as, nb_arr[nb_index]);
-    }
-
-    for (; sb_index < sb_n; sb_index++) {
-        sbrec_address_set_update_addresses_delvalue(sb_as, sb_arr[sb_index]);
-    }
+    struct sorted_array sb_addresses =
+        sorted_array_from_dbrec(sb_as, addresses);
+    sorted_array_apply_diff(nb_addresses, &sb_addresses,
+                            sb_addr_set_apply_diff, sb_as);
+    sorted_array_destroy(&sb_addresses);
 }
 
 static void
@@ -415,32 +508,3 @@ sb_address_set_lookup_by_name(struct ovsdb_idl_index *sbrec_addr_set_by_name,
 
     return retval;
 }
-
-static struct sorted_addresses
-sorted_addresses_from_nbrec(const struct nbrec_address_set *nb_as)
-{
-    /* The DB is already sorted. */
-    return (struct sorted_addresses) {
-        .arr = (const char **) nb_as->addresses,
-        .n = nb_as->n_addresses,
-    };
-}
-
-static struct sorted_addresses
-sorted_addresses_from_svec(struct svec *addresses)
-{
-    svec_sort(addresses);
-    return (struct sorted_addresses) {
-        .arr = (const char **) addresses->names,
-        .n = addresses->n,
-    };
-}
-
-static struct sorted_addresses
-sorted_addresses_from_sset(struct sset *addresses)
-{
-    return (struct sorted_addresses) {
-        .arr = sset_sort(addresses),
-        .n = sset_count(addresses),
-    };
-}
