@@ -26,14 +26,17 @@
 #include "lib/ovn-nb-idl.h"
 #include "lib/ovn-sb-idl.h"
 #include "mcast-group-index.h"
-#include "northd/mac-binding-aging.h"
+#include "northd/aging.h"
 #include "openvswitch/poll-loop.h"
 #include "openvswitch/vlog.h"
 #include "inc-proc-northd.h"
+#include "en-lb-data.h"
 #include "en-northd.h"
 #include "en-lflow.h"
 #include "en-northd-output.h"
+#include "en-meters.h"
 #include "en-sync-sb.h"
+#include "en-sync-from-sb.h"
 #include "unixctl.h"
 #include "util.h"
 
@@ -43,32 +46,15 @@ static unixctl_cb_func chassis_features_list;
 
 #define NB_NODES \
     NB_NODE(nb_global, "nb_global") \
-    NB_NODE(copp, "copp") \
     NB_NODE(logical_switch, "logical_switch") \
-    NB_NODE(logical_switch_port, "logical_switch_port") \
-    NB_NODE(forwarding_group, "forwarding_group") \
     NB_NODE(address_set, "address_set") \
     NB_NODE(port_group, "port_group") \
     NB_NODE(load_balancer, "load_balancer") \
     NB_NODE(load_balancer_group, "load_balancer_group") \
-    NB_NODE(load_balancer_health_check, "load_balancer_health_check") \
     NB_NODE(acl, "acl") \
     NB_NODE(logical_router, "logical_router") \
-    NB_NODE(qos, "qos") \
     NB_NODE(mirror, "mirror") \
     NB_NODE(meter, "meter") \
-    NB_NODE(meter_band, "meter_band") \
-    NB_NODE(logical_router_port, "logical_router_port") \
-    NB_NODE(logical_router_static_route, "logical_router_static_route") \
-    NB_NODE(logical_router_policy, "logical_router_policy") \
-    NB_NODE(nat, "nat") \
-    NB_NODE(dhcp_options, "dhcp_options") \
-    NB_NODE(connection, "connection") \
-    NB_NODE(dns, "dns") \
-    NB_NODE(ssl, "ssl") \
-    NB_NODE(gateway_chassis, "gateway_chassis") \
-    NB_NODE(ha_chassis_group, "ha_chassis_group") \
-    NB_NODE(ha_chassis, "ha_chassis") \
     NB_NODE(bfd, "bfd") \
     NB_NODE(static_mac_binding, "static_mac_binding") \
     NB_NODE(chassis_template_var, "chassis_template_var")
@@ -92,30 +78,17 @@ static unixctl_cb_func chassis_features_list;
 #define SB_NODES \
     SB_NODE(sb_global, "sb_global") \
     SB_NODE(chassis, "chassis") \
-    SB_NODE(chassis_private, "chassis_private") \
-    SB_NODE(encap, "encap") \
     SB_NODE(address_set, "address_set") \
     SB_NODE(port_group, "port_group") \
     SB_NODE(logical_flow, "logical_flow") \
-    SB_NODE(logical_dp_group, "logical_DP_group") \
     SB_NODE(multicast_group, "multicast_group") \
     SB_NODE(mirror, "mirror") \
     SB_NODE(meter, "meter") \
-    SB_NODE(meter_band, "meter_band") \
     SB_NODE(datapath_binding, "datapath_binding") \
     SB_NODE(port_binding, "port_binding") \
     SB_NODE(mac_binding, "mac_binding") \
-    SB_NODE(dhcp_options, "dhcp_options") \
-    SB_NODE(dhcpv6_options, "dhcpv6_options") \
-    SB_NODE(connection, "connection") \
-    SB_NODE(ssl, "ssl") \
     SB_NODE(dns, "dns") \
-    SB_NODE(rbac_role, "rbac_role") \
-    SB_NODE(rbac_permission, "rbac_permission") \
-    SB_NODE(gateway_chassis, "gateway_chassis") \
-    SB_NODE(ha_chassis, "ha_chassis") \
     SB_NODE(ha_chassis_group, "ha_chassis_group") \
-    SB_NODE(controller_event, "controller_event") \
     SB_NODE(ip_multicast, "ip_multicast") \
     SB_NODE(igmp_group, "igmp_group") \
     SB_NODE(service_monitor, "service_monitor") \
@@ -158,90 +131,87 @@ enum sb_engine_node {
 
 /* Define engine nodes for other nodes. They should be defined as static to
  * avoid sparse errors. */
-static ENGINE_NODE(northd, "northd");
+static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(northd, "northd");
+static ENGINE_NODE(sync_from_sb, "sync_from_sb");
 static ENGINE_NODE(lflow, "lflow");
 static ENGINE_NODE(mac_binding_aging, "mac_binding_aging");
 static ENGINE_NODE(mac_binding_aging_waker, "mac_binding_aging_waker");
 static ENGINE_NODE(northd_output, "northd_output");
+static ENGINE_NODE(sync_meters, "sync_meters");
 static ENGINE_NODE(sync_to_sb, "sync_to_sb");
 static ENGINE_NODE(sync_to_sb_addr_set, "sync_to_sb_addr_set");
+static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(port_group, "port_group");
+static ENGINE_NODE(fdb_aging, "fdb_aging");
+static ENGINE_NODE(fdb_aging_waker, "fdb_aging_waker");
+static ENGINE_NODE(sync_to_sb_lb, "sync_to_sb_lb");
+static ENGINE_NODE(sync_to_sb_pb, "sync_to_sb_pb");
+static ENGINE_NODE_WITH_CLEAR_TRACK_DATA(lb_data, "lb_data");
 
 void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
                           struct ovsdb_idl_loop *sb)
 {
     /* Define relationships between nodes where first argument is dependent
      * on the second argument */
-    engine_add_input(&en_northd, &en_nb_nb_global, NULL);
-    engine_add_input(&en_northd, &en_nb_copp, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_switch, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_switch_port, NULL);
-    engine_add_input(&en_northd, &en_nb_forwarding_group, NULL);
-    engine_add_input(&en_northd, &en_nb_port_group, NULL);
-    engine_add_input(&en_northd, &en_nb_load_balancer, NULL);
-    engine_add_input(&en_northd, &en_nb_load_balancer_group, NULL);
-    engine_add_input(&en_northd, &en_nb_load_balancer_health_check, NULL);
-    engine_add_input(&en_northd, &en_nb_acl, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_router, NULL);
-    engine_add_input(&en_northd, &en_nb_qos, NULL);
+    engine_add_input(&en_lb_data, &en_nb_load_balancer,
+                     lb_data_load_balancer_handler);
+    engine_add_input(&en_lb_data, &en_nb_load_balancer_group,
+                     lb_data_load_balancer_group_handler);
+    engine_add_input(&en_lb_data, &en_nb_logical_switch,
+                     lb_data_logical_switch_handler);
+    engine_add_input(&en_lb_data, &en_nb_logical_router,
+                     lb_data_logical_router_handler);
+
     engine_add_input(&en_northd, &en_nb_mirror, NULL);
-    engine_add_input(&en_northd, &en_nb_meter, NULL);
-    engine_add_input(&en_northd, &en_nb_meter_band, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_router_port, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_router_static_route, NULL);
-    engine_add_input(&en_northd, &en_nb_logical_router_policy, NULL);
-    engine_add_input(&en_northd, &en_nb_nat, NULL);
-    engine_add_input(&en_northd, &en_nb_dhcp_options, NULL);
-    engine_add_input(&en_northd, &en_nb_connection, NULL);
-    engine_add_input(&en_northd, &en_nb_dns, NULL);
-    engine_add_input(&en_northd, &en_nb_ssl, NULL);
-    engine_add_input(&en_northd, &en_nb_gateway_chassis, NULL);
-    engine_add_input(&en_northd, &en_nb_ha_chassis_group, NULL);
-    engine_add_input(&en_northd, &en_nb_ha_chassis, NULL);
     engine_add_input(&en_northd, &en_nb_static_mac_binding, NULL);
     engine_add_input(&en_northd, &en_nb_chassis_template_var, NULL);
 
     engine_add_input(&en_northd, &en_sb_sb_global, NULL);
     engine_add_input(&en_northd, &en_sb_chassis, NULL);
-    engine_add_input(&en_northd, &en_sb_chassis_private, NULL);
-    engine_add_input(&en_northd, &en_sb_encap, NULL);
-    engine_add_input(&en_northd, &en_sb_port_group, NULL);
-    engine_add_input(&en_northd, &en_sb_logical_dp_group, NULL);
     engine_add_input(&en_northd, &en_sb_mirror, NULL);
     engine_add_input(&en_northd, &en_sb_meter, NULL);
-    engine_add_input(&en_northd, &en_sb_meter_band, NULL);
     engine_add_input(&en_northd, &en_sb_datapath_binding, NULL);
-    engine_add_input(&en_northd, &en_sb_port_binding, NULL);
     engine_add_input(&en_northd, &en_sb_mac_binding, NULL);
-    engine_add_input(&en_northd, &en_sb_dhcp_options, NULL);
-    engine_add_input(&en_northd, &en_sb_dhcpv6_options, NULL);
-    engine_add_input(&en_northd, &en_sb_connection, NULL);
-    engine_add_input(&en_northd, &en_sb_ssl, NULL);
     engine_add_input(&en_northd, &en_sb_dns, NULL);
-    engine_add_input(&en_northd, &en_sb_rbac_role, NULL);
-    engine_add_input(&en_northd, &en_sb_rbac_permission, NULL);
-    engine_add_input(&en_northd, &en_sb_gateway_chassis, NULL);
-    engine_add_input(&en_northd, &en_sb_ha_chassis, NULL);
     engine_add_input(&en_northd, &en_sb_ha_chassis_group, NULL);
-    engine_add_input(&en_northd, &en_sb_controller_event, NULL);
     engine_add_input(&en_northd, &en_sb_ip_multicast, NULL);
     engine_add_input(&en_northd, &en_sb_service_monitor, NULL);
-    engine_add_input(&en_northd, &en_sb_load_balancer, NULL);
     engine_add_input(&en_northd, &en_sb_fdb, NULL);
     engine_add_input(&en_northd, &en_sb_static_mac_binding, NULL);
     engine_add_input(&en_northd, &en_sb_chassis_template_var, NULL);
+
+    engine_add_input(&en_northd, &en_sb_port_binding,
+                     northd_sb_port_binding_handler);
+    engine_add_input(&en_northd, &en_nb_nb_global,
+                     northd_nb_nb_global_handler);
+    engine_add_input(&en_northd, &en_nb_logical_switch,
+                     northd_nb_logical_switch_handler);
+    engine_add_input(&en_northd, &en_nb_logical_router,
+                     northd_nb_logical_router_handler);
+    engine_add_input(&en_northd, &en_lb_data, northd_lb_data_handler);
+
     engine_add_input(&en_mac_binding_aging, &en_nb_nb_global, NULL);
     engine_add_input(&en_mac_binding_aging, &en_sb_mac_binding, NULL);
     engine_add_input(&en_mac_binding_aging, &en_northd, NULL);
     engine_add_input(&en_mac_binding_aging, &en_mac_binding_aging_waker, NULL);
+
+    engine_add_input(&en_fdb_aging, &en_nb_nb_global, NULL);
+    engine_add_input(&en_fdb_aging, &en_sb_fdb, NULL);
+    engine_add_input(&en_fdb_aging, &en_northd, NULL);
+    engine_add_input(&en_fdb_aging, &en_fdb_aging_waker, NULL);
+
+    engine_add_input(&en_sync_meters, &en_nb_acl, NULL);
+    engine_add_input(&en_sync_meters, &en_nb_meter, NULL);
+    engine_add_input(&en_sync_meters, &en_sb_meter, NULL);
+
     engine_add_input(&en_lflow, &en_nb_bfd, NULL);
+    engine_add_input(&en_lflow, &en_nb_acl, NULL);
+    engine_add_input(&en_lflow, &en_sync_meters, NULL);
     engine_add_input(&en_lflow, &en_sb_bfd, NULL);
     engine_add_input(&en_lflow, &en_sb_logical_flow, NULL);
     engine_add_input(&en_lflow, &en_sb_multicast_group, NULL);
     engine_add_input(&en_lflow, &en_sb_igmp_group, NULL);
-    engine_add_input(&en_lflow, &en_northd, NULL);
-    /* XXX: The "en_mac_binding_aging" should be separate "root" node
-     * once I-P engine allows multiple root nodes. */
-    engine_add_input(&en_lflow, &en_mac_binding_aging, NULL);
+    engine_add_input(&en_lflow, &en_northd, lflow_northd_handler);
+    engine_add_input(&en_lflow, &en_port_group, lflow_port_group_handler);
 
     engine_add_input(&en_sync_to_sb_addr_set, &en_nb_address_set,
                      sync_to_sb_addr_set_nb_address_set_handler);
@@ -250,15 +220,48 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_add_input(&en_sync_to_sb_addr_set, &en_northd, NULL);
     engine_add_input(&en_sync_to_sb_addr_set, &en_sb_address_set, NULL);
 
+    engine_add_input(&en_port_group, &en_nb_port_group,
+                     port_group_nb_port_group_handler);
+    engine_add_input(&en_port_group, &en_sb_port_group, NULL);
+    /* No need for an explicit handler for northd changes.  Port changes
+     * that affect port_groups trigger updates to the NB.Port_Group
+     * table too (because of the explicit dependency in the schema). */
+    engine_add_input(&en_port_group, &en_northd, engine_noop_handler);
+
+    engine_add_input(&en_sync_to_sb_lb, &en_northd,
+                     sync_to_sb_lb_northd_handler);
+    engine_add_input(&en_sync_to_sb_lb, &en_sb_load_balancer,
+                     sync_to_sb_lb_sb_load_balancer);
+
+    engine_add_input(&en_sync_to_sb_pb, &en_northd,
+                     sync_to_sb_pb_northd_handler);
+
     /* en_sync_to_sb engine node syncs the SB database tables from
      * the NB database tables.
-     * Right now this engine only syncs the SB Address_Set table.
+     * Right now this engine syncs the SB Address_Set table, Port_Group table
+     * SB Meter/Meter_Band tables and SB Load_Balancer table and
+     * (partly) SB Port_Binding table.
      */
     engine_add_input(&en_sync_to_sb, &en_sync_to_sb_addr_set, NULL);
+    engine_add_input(&en_sync_to_sb, &en_port_group, NULL);
+    engine_add_input(&en_sync_to_sb, &en_sync_meters, NULL);
+    engine_add_input(&en_sync_to_sb, &en_sync_to_sb_lb, NULL);
+    engine_add_input(&en_sync_to_sb, &en_sync_to_sb_pb, NULL);
+
+    engine_add_input(&en_sync_from_sb, &en_northd,
+                     sync_from_sb_northd_handler);
+    engine_add_input(&en_sync_from_sb, &en_sb_port_binding, NULL);
+    engine_add_input(&en_sync_from_sb, &en_sb_ha_chassis_group, NULL);
+
+    engine_add_input(&en_northd_output, &en_sync_from_sb, NULL);
     engine_add_input(&en_northd_output, &en_sync_to_sb,
                      northd_output_sync_to_sb_handler);
     engine_add_input(&en_northd_output, &en_lflow,
                      northd_output_lflow_handler);
+    engine_add_input(&en_northd_output, &en_mac_binding_aging,
+                     northd_output_mac_binding_aging_handler);
+    engine_add_input(&en_northd_output, &en_fdb_aging,
+                     northd_output_fdb_aging_handler);
 
     struct engine_arg engine_arg = {
         .nb_idl = nb->idl,
@@ -279,6 +282,8 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
         = static_mac_binding_index_create(sb->idl);
     struct ovsdb_idl_index *sbrec_mac_binding_by_datapath
         = mac_binding_by_datapath_index_create(sb->idl);
+    struct ovsdb_idl_index *fdb_by_dp_key =
+        ovsdb_idl_index_create1(sb->idl, &sbrec_fdb_col_dp_key);
 
     engine_init(&en_northd_output, &engine_arg);
 
@@ -303,6 +308,9 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
     engine_ovsdb_node_add_index(&en_sb_mac_binding,
                                 "sbrec_mac_binding_by_datapath",
                                 sbrec_mac_binding_by_datapath);
+    engine_ovsdb_node_add_index(&en_sb_fdb,
+                                "fdb_by_dp_key",
+                                fdb_by_dp_key);
 
     struct ovsdb_idl_index *sbrec_address_set_by_name
         = ovsdb_idl_index_create1(sb->idl, &sbrec_address_set_col_name);
@@ -310,6 +318,19 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
                                 "sbrec_address_set_by_name",
                                 sbrec_address_set_by_name);
 
+    struct ovsdb_idl_index *sbrec_port_group_by_name
+        = ovsdb_idl_index_create1(sb->idl, &sbrec_port_group_col_name);
+    engine_ovsdb_node_add_index(&en_sb_port_group,
+                                "sbrec_port_group_by_name",
+                                sbrec_port_group_by_name);
+
+    struct ovsdb_idl_index *sbrec_fdb_by_dp_and_port
+        = ovsdb_idl_index_create2(sb->idl, &sbrec_fdb_col_dp_key,
+                                  &sbrec_fdb_col_port_key);
+    engine_ovsdb_node_add_index(&en_sb_fdb,
+                                "sbrec_fdb_by_dp_and_port",
+                                sbrec_fdb_by_dp_and_port);
+
     struct northd_data *northd_data =
         engine_get_internal_data(&en_northd);
     unixctl_command_register("debug/chassis-features-list", "", 0, 0,
@@ -320,15 +341,18 @@ void inc_proc_northd_init(struct ovsdb_idl_loop *nb,
 /* Returns true if the incremental processing ended up updating nodes. */
 bool inc_proc_northd_run(struct ovsdb_idl_txn *ovnnb_txn,
                          struct ovsdb_idl_txn *ovnsb_txn,
-                         bool recompute) {
+                         struct northd_engine_context *ctx) {
+    ovs_assert(ovnnb_txn && ovnsb_txn);
+
+    int64_t start = time_msec();
     engine_init_run();
 
     /* Force a full recompute if instructed to, for example, after a NB/SB
      * reconnect event.  However, make sure we don't overwrite an existing
      * force-recompute request if 'recompute' is false.
      */
-    if (recompute) {
-        engine_set_force_recompute(recompute);
+    if (ctx->recompute) {
+        engine_set_force_recompute(ctx->recompute);
     }
 
     struct engine_context eng_ctx = {
@@ -337,10 +361,7 @@ bool inc_proc_northd_run(struct ovsdb_idl_txn *ovnnb_txn,
     };
 
     engine_set_context(&eng_ctx);
-
-    if (ovnnb_txn && ovnsb_txn) {
-        engine_run(true);
-    }
+    engine_run(true);
 
     if (!engine_has_run()) {
         if (engine_need_run()) {
@@ -357,6 +378,12 @@ bool inc_proc_northd_run(struct ovsdb_idl_txn *ovnnb_txn,
     } else {
         engine_set_force_recompute(false);
     }
+
+    int64_t now = time_msec();
+    /* Postpone the next run by length of current run with maximum capped
+     * by "northd-backoff-interval-ms" interval. */
+    ctx->next_run_ms = now + MIN(now - start, ctx->backoff_ms);
+
     return engine_has_updated();
 }
 
@@ -366,6 +393,19 @@ void inc_proc_northd_cleanup(void)
     engine_set_context(NULL);
 }
 
+bool
+inc_proc_northd_can_run(struct northd_engine_context *ctx)
+{
+    if (ctx->recompute || time_msec() >= ctx->next_run_ms ||
+        ctx->nb_idl_duration_ms >= IDL_LOOP_MAX_DURATION_MS ||
+        ctx->sb_idl_duration_ms >= IDL_LOOP_MAX_DURATION_MS) {
+        return true;
+    }
+
+    poll_timer_wait_until(ctx->next_run_ms);
+    return false;
+}
+
 static void
 chassis_features_list(struct unixctl_conn *conn, int argc OVS_UNUSED,
                       const char *argv[] OVS_UNUSED, void *features_)
